/** @file fifo.cpp

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.09.17-

	@brief [ fifo buffer ]
*/

#include "fifo.h"
#include <stdlib.h>

FIFO::FIFO(int s)
{
		cnt = rpt = wpt = 0;
		size = s;
		buf = (int*)malloc(s * sizeof(int));
}
void FIFO::release()
{
	free(buf);
}
void FIFO::clear()
{
	cnt = rpt = wpt = 0;
}
void FIFO::write(int val)
{
	if(cnt < size) {
		buf[wpt++] = val;
		if(wpt >= size) {
			wpt = 0;
		}
		cnt++;
	}
}
int FIFO::read()
{
	int val = 0;
	if(cnt) {
		val = buf[rpt++];
		if(rpt >= size) {
			rpt = 0;
		}
		cnt--;
	}
	return val;
}
int FIFO::read_not_remove(int pt)
{
	if(pt >= 0 && pt < cnt) {
		pt += rpt;
		if(pt >= size) {
			pt -= size;
		}
		return buf[pt];
	}
	return 0;
}
int FIFO::count()
{
	return cnt;
}
bool FIFO::full()
{
	return (cnt == size);
}
bool FIFO::empty()
{
	return (cnt == 0);
}

