/** @file ag_config_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.04.08

	@brief [ ag_config_dlg ]
*/

#include "ag_config_dlg.h"
#include <agar/core/types.h>
#include "../gui.h"
//#include "../../config.h"
#include "ag_gui_base.h"
//#include "../../clocale.h"
#include "../../msgboard.h"
#include "../../clocale.h"
#include "../../utility.h"

namespace GUI_AGAR
{

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

#if defined(_MBS1)
#define CONFIG_TABS	6
#else
#define CONFIG_TABS	5
#endif

AG_CONFIG_DLG::AG_CONFIG_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(&param, 0, sizeof(param));
	param.fdd_type  = config.fdd_type;
	param.io_port = config.io_port;
#if defined(_MBS1)
	param.sys_mode_rev = (1 - (config.sys_mode & 1));
#endif
	param.wav_reverse = 0;
	param.wav_half = 1;
	param.wav_correct = 0;

	param.wav_sample_rate = 3;
	param.wav_sample_bits = 0;
#if defined(_MBS1)
	param.disptmg_skew = config.disptmg_skew + 2;
	param.curdisp_skew = config.curdisp_skew + 2;
#else
	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;
#endif
	param.gl_filter_type = config.gl_filter_type;
	param.led_show = 0;
	param.led_pos = config.led_pos;
	param.capture_type = config.capture_type;
#if defined(_MBS1)
	param.exram_num = config.exram_size_num;
#endif

	paramtmp = NULL;

	dirdlg = new AG_DIR_DLG(parent_emu, parent_gui);
}

AG_CONFIG_DLG::~AG_CONFIG_DLG()
{
	delete dirdlg;
	delete paramtmp;
}

/**
 * create config dialog
 */
void AG_CONFIG_DLG::Create()
{
#if defined(_MBS1)
	param.disptmg_skew = config.disptmg_skew + 2;
	param.curdisp_skew = config.curdisp_skew + 2;
#else
	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;
#endif
	param.use_opengl = config.use_opengl;
	param.gl_filter_type = config.gl_filter_type;
	param.led_show = gui->GetLedBoxPhase(-1);
	param.led_pos = config.led_pos;
	param.capture_type = config.capture_type;
#if defined(_MBS1)
	param.sys_mode_rev = (1 - (emu->get_parami(VM::ParamSysMode) & 1));
#else
	param.exram_size_num = config.exram_size_num;
#endif

	param.use_power_off = (config.use_power_off ? 1 : 0);
	param.dipswitch = config.dipswitch;
	param.fdd_type = emu->get_parami(VM::ParamFddType);
	param.mount_fdd = config.mount_fdd;
	param.delayfd1 = FLG_DELAY_FDSEARCH ? 1 : 0;
	param.delayfd2 = FLG_DELAY_FDSEEK ? 1 : 0;
	param.io_port = emu->get_parami(VM::ParamIOPort);
	param.wav_reverse = config.wav_reverse ? 1 : 0;
	param.wav_half = config.wav_half ? 1 : 0;
	param.wav_correct = config.wav_correct ? config.wav_correct_type + 1 : 0;
	sprintf(param.wav_correct_amp[0], "%d", config.wav_correct_amp[0]);
	sprintf(param.wav_correct_amp[1], "%d", config.wav_correct_amp[1]);
	param.wav_sample_rate = config.wav_sample_rate;
	param.wav_sample_bits = config.wav_sample_bits;

	memcpy(param.snap_path, config.snapshot_path.Get(), _MAX_PATH);
	memcpy(param.rom_path, config.rom_path.Get(), _MAX_PATH);
	memcpy(param.font_path, config.font_path.Get(), _MAX_PATH);
	memcpy(param.msg_font_name, config.msgboard_msg_fontname.Get(), _MAX_PATH);
	sprintf(param.msg_font_size, "%d", config.msgboard_msg_fontsize);
	memcpy(param.info_font_name, config.msgboard_info_fontname.Get(), _MAX_PATH);
	sprintf(param.info_font_size, "%d", config.msgboard_info_fontsize);
	memcpy(param.menu_font_name, config.menu_fontname.Get(), _MAX_PATH);
	sprintf(param.menu_font_size, "%d", config.menu_fontsize);

	for(int i=0; i<MAX_PRINTER; i++) {
		memcpy(param.prn_host[i], config.printer_server_host[i].Get(), _MAX_PATH);
		sprintf(param.prn_port[i], "%d", config.printer_server_port[i]);
	}
	for(int i=0; i<MAX_COMM; i++) {
		memcpy(param.com_host[i], config.comm_server_host[i].Get(), _MAX_PATH);
		sprintf(param.com_port[i], "%d", config.comm_server_port[i]);
		param.com_dipsw[i] = config.comm_dipswitch[i];
	}
#ifdef USE_DEBUGGER
	memcpy(param.dbgr_host, config.debugger_server_host.Get(), _MAX_PATH);
	sprintf(param.dbgr_port, "%d", config.debugger_server_port);
#endif

#if defined(_MBS1)
	param.exram_num = emu->get_parami(VM::ParamExMemNum);
	param.mem_nowait = config.mem_nowait ? 1 : 0;

	param.fmopn_clock_num = config.opn_clock;
	param.fmopn_irq = config.opn_irq;
	param.use_opn_expsg = config.use_opn_expsg ? 1 : 0;
#if defined(USE_Z80B_CARD)
	param.z80b_card_out_irq = config.z80b_card_out_irq;
#endif
#endif
	param.showmsg_undefop = FLG_SHOWMSG_UNDEFOP ? 1 : 0;

	if (paramtmp == NULL) {
		paramtmp = new ConfigDlgParam;
	}
	*paramtmp = param;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, CMSG(Configure));

	AG_Box *vbox_base;
	AG_Box *hbox_base;
	AG_Box *vbox_left;
	AG_Box *vbox_right;
	AG_Box *vbox;
	AG_Box *hbox;
	AG_Radio *rad;
	AG_Checkbox *cbox;
	AG_Notebook *nb;
	AG_NotebookTab *tabs[CONFIG_TABS];

	int i;
	char name[128];

	nb = AG_NotebookNew(win, AG_NOTEBOOK_HFILL);

	//
	// 0 Mode
	tabs[0] = AG_NotebookAddTab(nb, CMSG(Mode), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[0], AG_BOX_HFILL | AG_BOX_VFILL);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// left
	vbox_left = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

#if defined(_MBS1)
	// System Mode
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(System_Mode_ASTERISK));

	set_label(radSysModeBase[0],CMSG(A_Mode_S1));
	set_label(radSysModeBase[1],CMSG(B_Mode_L3));
	for(i=0; i<2; i++) {
		radSysMode[i] = radSysModeBase[i];
		if (i == (1 - (config.sys_mode & 1))) {
			radSysModeBase[i][0] = '>';
		}
	}
	radSysMode[2]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radSysMode, &paramtmp->sys_mode_rev);

	strcpy(chkDipSwitchBase[0],CMSG(NEWON7));
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	cbox = AG_CheckboxNewFlag(hbox, 0, chkDipSwitchBase[0], (Uint *)&paramtmp->dipswitch, 0x04);
#else
	// Mode switch
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(DIP_Switch_ASTERISK));

	set_label(chkDipSwitchBase[0],CMSG(MODE_Switch));
	if (config.dipswitch & 0x04) {
		chkDipSwitchBase[0][0] = '>';
	}
	cbox = AG_CheckboxNewFlag(vbox, 0, chkDipSwitchBase[0], (Uint *)&paramtmp->dipswitch, 0x04);
#endif

	// FDD type
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(FDD_Type_ASTERISK));

	const CMsg::Id fdd_type_list[] = {
		CMsg::Non_FDD,
#if defined(_MBS1)
		CMsg::FD3inch_compact_FDD_L3,
		CMsg::FD5inch_mini_FDD_2D_Type,
		CMsg::FD5inch_mini_FDD_2HD_Type,
#else
		CMsg::FD3inch_compact_FDD,
		CMsg::FD5inch_mini_FDD,
		CMsg::FD8inch_standard_FDD,
#endif
		CMsg::End
	};
	for(i=0; i<4; i++) {
		set_label(radFddTypeBase[i], fdd_type_list[i]);
	}
	for(i=0; i<4; i++) {
		radFddType[i] = radFddTypeBase[i];
		if (i == config.fdd_type) {
			radFddTypeBase[i][0] = '>';
		}
	}
	radFddType[4]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radFddType, &paramtmp->fdd_type);
	AG_AddEvent(rad, "radio-changed", OnChangeFddType, "%Cp", this);

	AG_CheckboxNewInt(vbox_left, 0, CMSG(Enable_the_state_of_power_off), &paramtmp->use_power_off);

	// right
	vbox_right = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// I/O Port address
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(I_O_Port_Address_ASTERISK));

	const CMsg::Id io_port_list[] = { 
#if defined(_MBS1)
		CMsg::FDC5__FF00,
		CMsg::FDC3L_FF18,
		CMsg::PSG6L_FF30,
		CMsg::ExLPT_FF3C,
		CMsg::ExCOM_FF40,
		CMsg::PSG9L_FF70,
		CMsg::KANJI_FF75,
		CMsg::ExPSG_FFE6,
		CMsg::OS9Ex_FE11,
		CMsg::Comm__FF77,
		CMsg::Keybd_FFE0,
		CMsg::Mouse_FFDC,
		CMsg::FMSyn_FF1E,
		CMsg::DisROM_FFCE,
		CMsg::DisIGL_FFE9,
		CMsg::RTC___FF38,
#if defined(USE_Z80B_CARD)
		CMsg::Z80BCD_FF7F,
#endif
#else
		CMsg::FDC5__FF00,
		CMsg::FDC3__FF18,
		CMsg::PSG6__FF30,
		CMsg::ExLPT_FF3C,
		CMsg::ExCOM_FF40,
		CMsg::PSG9__FF70,
		CMsg::KANJI_FF75,
		CMsg::Null,
		CMsg::Null,
		CMsg::Null,
		CMsg::Null,
		CMsg::Null,
		CMsg::Null,
		CMsg::Null,
		CMsg::DisIG_FFE9,
		CMsg::RTC___FF38,
#endif
		CMsg::End
	};
	for(i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if ((1 << i) & IOPORT_MSK_ALL) {
			set_label(chkIOPortBase[i], io_port_list[i]);
		}
	}
	for(i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if ((1 << i) & IOPORT_MSK_ALL) {
			if (config.io_port & (1 << i)) {
				chkIOPortBase[i][0] = '>';
			}
			cbox = AG_CheckboxNewFlag(vbox, 0, chkIOPortBase[i], (Uint *)&paramtmp->io_port, (1 << i));
			AG_AddEvent(cbox, "checkbox-changed", OnChangeIOPort, "%Cp %i", this, i);
			chkIOPort[i] = cbox;
		} else {
			chkIOPort[i] = NULL;
		}
	}

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, CMSG(Need_restart_program_or_PowerOn));


	//
	// 1 screen
	tabs[1] = AG_NotebookAddTab(nb, CMSG(Screen), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[1], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

#ifdef USE_OPENGL
	// GL Filter Type
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(OpenGL));

	const CMsg::Id gl_use_list[] = {
		CMsg::No,
		CMsg::Yes_sync,
		CMsg::Yes_async,
		CMsg::End
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Use_ASTERISK));
	UComboNew(hbox, gl_use_list, config.use_opengl, OnChangeUseOpenGL);

	const CMsg::Id gl_filter_list[] = {
		CMsg::Nearest_Neighbour,
		CMsg::Linear,
		CMsg::End
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Filter_Type));
	UComboNew(hbox, gl_filter_list, config.gl_filter_type, OnChangeGLFilterType);
#endif

	// CRTC
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(CRTC));

	const char *skew_list[] = {
#if defined(_MBS1)
		"-2", "-1", "0", "1", "2", NULL
#else
		"0", "1", "2", NULL
#endif
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Disptmg_Skew));
#if defined(_MBS1)
	UComboNew(hbox, skew_list, config.disptmg_skew + 2, OnChangeDisptmgSkew);
#else
	UComboNew(hbox, skew_list, config.disptmg_skew, OnChangeDisptmgSkew);
#endif

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
#if defined(_MBS1)
	AG_LabelNewS(hbox, 0, CMSG(Curdisp_Skew_L3));
	UComboNew(hbox, skew_list, config.curdisp_skew + 2, OnChangeCurdispSkew);
#else
	AG_LabelNewS(hbox, 0, CMSG(Curdisp_Skew));
	UComboNew(hbox, skew_list, config.curdisp_skew, OnChangeCurdispSkew);
#endif

	// LED
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	const CMsg::Id led_show_list[] = {
		CMsg::Hide,
		CMsg::Show_Inside,
		CMsg::Show_Outside,
		CMsg::End
	};
	vbox = AG_BoxNewVert(hbox_base, 0);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(LED));
	UComboNew(hbox, led_show_list, paramtmp->led_show, OnChangeLedShow);

	const CMsg::Id led_pos_list[] = {
		CMsg::LeftTop,
		CMsg::RightTop,
		CMsg::LeftBottom,
		CMsg::RightBottom,
		CMsg::End
	};
	vbox = AG_BoxNewVert(hbox_base, 0);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Position));
	UComboNew(hbox, led_pos_list, config.led_pos, OnChangeLedPos);

	// capture type
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	const char *cap_type_list[] = {
		"BMP",
#if defined(USE_REC_VIDEO_WIN) || defined(USE_REC_VIDEO_COCOA) || defined(USE_REC_VIDEO_LIBPNG)
		"PNG",
#endif
		NULL
	};
	AG_LabelNewS(hbox, 0, CMSG(Capture_Type));
	UComboNew(hbox, cap_type_list, config.capture_type, OnChangeCaptureType);

	// snapshot path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(Snapshot_Path), 40, paramtmp->snap_path, sizeof(paramtmp->snap_path)-1);
	AG_ButtonNewFn(hbox, 0, CMSG(Folder_), OnShowDirDlg, "%Cp %Cp %i", this, win, SnapShotPath);

	// font path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(Font_Path), 40, paramtmp->font_path, sizeof(paramtmp->font_path)-1);
	AG_ButtonNewFn(hbox, 0, CMSG(Folder_), OnShowDirDlg, "%Cp %Cp %i", this, win, FontPath);

	// msgboard font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(Message_Font), 16, paramtmp->msg_font_name, sizeof(paramtmp->msg_font_name)-1);
	TextboxNew(hbox, CMSG(_Size), 4, paramtmp->msg_font_size, sizeof(paramtmp->msg_font_size)-1);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(Info_Font), 16, paramtmp->info_font_name, sizeof(paramtmp->info_font_name)-1);
	TextboxNew(hbox, CMSG(_Size), 4, paramtmp->info_font_size, sizeof(paramtmp->info_font_size)-1);

	// menu font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(Menu_Font_ASTERISK), 16, paramtmp->menu_font_name, sizeof(paramtmp->menu_font_name)-1);
	TextboxNew(hbox, CMSG(_Size), 4, paramtmp->menu_font_size, sizeof(paramtmp->menu_font_size)-1);

	// language
	lang_list.Clear();
	clocale->GetLocaleNamesWithDefault(lang_list);

	paramtmp->lang_selidx = clocale->SelectLocaleNameIndex(lang_list, config.language);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Language_ASTERISK));
	UComboNew(hbox, lang_list, paramtmp->lang_selidx, OnChangeLanguage);

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, CMSG(Need_restart_program));

	//
	// 2 tape
	tabs[2] = AG_NotebookAddTab(nb, CMSG(Tape_FDD), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[2], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// Load wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(Load_Wav_File_from_Tape));

	AG_CheckboxNewInt(vbox, 0, CMSG(Reverse_Wave), &paramtmp->wav_reverse);
	AG_CheckboxNewInt(vbox, 0, CMSG(Half_Wave), &paramtmp->wav_half);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Correct));

	const CMsg::Id radCorrect[] = {
		CMsg::None_,
		CMsg::COS_Wave,
		CMsg::SIN_Wave,
		CMsg::End
	};
	rad = RadioNewInt(hbox, AG_RADIO_EXPAND, radCorrect, &paramtmp->wav_correct);

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	TextboxNew(hbox, "1200Hz", 6, paramtmp->wav_correct_amp[0], sizeof(paramtmp->wav_correct_amp[0])-1);
	TextboxNew(hbox, "2400Hz", 6, paramtmp->wav_correct_amp[1], sizeof(paramtmp->wav_correct_amp[1])-1);

//	AG_AddEvent(cbox, "checkbox-changed", OnChangeCorrect, "%Cp %Cp", this, rad);
//	if (paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}

	// Save wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(Save_Wav_File_to_Tape));

	const char *rate_list[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Sample_Rate));
	UComboNew(hbox, rate_list, paramtmp->wav_sample_rate, OnChangeSampleRate);

	const char *bits_list[] = {
		"8",
		"16",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Sample_Bits));
	UComboNew(hbox, bits_list, paramtmp->wav_sample_bits, OnChangeSampleBits);

	// FDD
	vbox = AG_BoxNewVert(vbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, CMSG(Floppy_Disk_Drive));

	// fdd mount
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(When_start_up_mount_disk_at_));
	for(i=0; i<MAX_DRIVE; i++) {
		sprintf(name, "%d  ", i);
		AG_CheckboxNewFlag(hbox, 0, name, (Uint *)&paramtmp->mount_fdd, (1 << i));
	}
	// delay
	AG_CheckboxNewInt(vbox, 0, CMSG(Ignore_delays_to_find_sector), &paramtmp->delayfd1);
	AG_CheckboxNewInt(vbox, 0, CMSG(Ignore_delays_to_seek_track), &paramtmp->delayfd2);

	//
	// 3 network
	tabs[3] = AG_NotebookAddTab(nb, CMSG(Network), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[3], AG_BOX_EXPAND);

	// LPT
	for(i=0; i<MAX_PRINTER; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, CMSG(LPTVDIGIT_Hostname), i);
		TextboxNew(hbox, name, 15, paramtmp->prn_host[i], sizeof(paramtmp->prn_host[i])-1);
		TextboxNew(hbox, CMSG(_Port), 6, paramtmp->prn_port[i], sizeof(paramtmp->prn_port[i])-1);
	}
	// COM
	const CMsg::Id comm_baud_list[] = {
		CMsg::S_300baud_F_1200baud,
		CMsg::S_600baud_F_2400baud,
		CMsg::S_1200baud_F_4800baud,
		CMsg::S_2400baud_F_9600baud,
		CMsg::End
	};
	for(i=0; i<MAX_COMM; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, CMSG(COMVDIGIT_Hostname), i);
		TextboxNew(hbox, name, 15, paramtmp->com_host[i], sizeof(paramtmp->com_host[i])-1);
		TextboxNew(hbox, CMSG(_Port), 6, paramtmp->com_port[i], sizeof(paramtmp->com_port[i])-1);

		UComboNew(hbox, comm_baud_list, paramtmp->com_dipsw[i]-1, OnChangeCommBaud, i);
	}
#ifdef USE_DEBUGGER
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Connectable_host_to_Debugger));
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _T(" "), 15, paramtmp->dbgr_host, sizeof(paramtmp->dbgr_host)-1);
	TextboxNew(hbox, CMSG(_Port), 6, paramtmp->dbgr_port, sizeof(paramtmp->dbgr_port)-1);
#endif

	//
	// 4 CPU, Memory
	tabs[4] = AG_NotebookAddTab(nb, CMSG(CPU_Memory), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[4], AG_BOX_EXPAND);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, CMSG(ROM_Path_ASTERISK), 40, paramtmp->rom_path, sizeof(paramtmp->rom_path)-1);
	AG_ButtonNewFn(hbox, 0, CMSG(Folder_), OnShowDirDlg, "%Cp %Cp %i", this, win, RomPath);
#if defined(_MBS1)
	// extended memory
	const CMsg::Id exram_list[] = {
		CMsg::None_, CMsg::S64KB, CMsg::S128KB, CMsg::S256KB, CMsg::S512KB, CMsg::End
	};
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Extended_RAM_ASTERISK));
	UComboNew(hbox, exram_list, paramtmp->exram_num, OnChangeExtRam);
	if (0 <= config.exram_size_num && config.exram_size_num <= 4) {
		_TCHAR str[100];
		UTILITY::tcscpy(str, 100, CMSG(LB_Now_SP));
		UTILITY::tcscat(str, 100, CMSGV(exram_list[config.exram_size_num]));
		UTILITY::tcscat(str, 100, _T(")"));
		AG_LabelNewS(hbox, 0, str);
	}
	// memory wait
	AG_CheckboxNewInt(vbox_base, 0, CMSG(No_wait_to_access_the_main_memory), &paramtmp->mem_nowait);
#else
	// extended memory
	AG_CheckboxNewFlag(vbox_base, 0, CMSG(Use_Extended_Memory_64KB), (Uint *)&paramtmp->exram_size_num, 1);
#endif
	//
	AG_CheckboxNewInt(vbox_base, 0, CMSG(Show_message_when_the_CPU_fetches_undefined_opcode), &paramtmp->showmsg_undefop);

#if defined(_MBS1) && defined(USE_Z80B_CARD)
	// Z80B Card
	const CMsg::Id z80bcard_irq_list[] = {
		CMsg::IRQ, CMsg::NMI, CMsg::End
	};
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Connect_interrupt_signal_of_Z80B_Card_to_ASTERISK));
	UComboNew(hbox, z80bcard_irq_list, paramtmp->z80b_card_out_irq, OnChangeZ80BCardIrq);
#endif

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, CMSG(Need_restart_program_or_PowerOn));

#if defined(_MBS1)
	//
	// 5 Sound
	tabs[5] = AG_NotebookAddTab(nb, CMSG(Sound), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[5], AG_BOX_EXPAND);

	const char *fmopn_clock_list[] = {
		_T("3,579,545"), _T("1,008,000"), _T("2,016,000"), _T("4,032,000"), NULL
	};
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Clock_of_FM_Synthesis_Card_ASTERISK));
	UComboNew(hbox, fmopn_clock_list, paramtmp->fmopn_clock_num, OnChangeFmOpnClock);
	AG_LabelNewS(hbox, 0, _T("Hz"));
	//
	const CMsg::Id fmopn_irq_list[] = {
		CMsg::None_, CMsg::IRQ, CMsg::FIRQ, CMsg::End
	};
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, CMSG(Connect_interrupt_signal_of_FM_Synthesis_to_ASTERISK));
	UComboNew(hbox, fmopn_irq_list, paramtmp->fmopn_irq, OnChangeFmOpnIrq);
	//
	AG_CheckboxNewInt(vbox_base, 0, CMSG(Use_FM_Synthesis_on_Extended_PSG_port), &paramtmp->use_opn_expsg);

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, CMSG(Need_restart_program));
#endif

	//
	// button
	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox_base, 0, CMSG(OK), OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox_base, 0, CMSG(Cancel), OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->PostEtSystemPause(true);
	AG_WindowShow(win);

	//
	change_fdd_type(paramtmp->fdd_type);
}

void AG_CONFIG_DLG::change_fdd_type(int index)
{
	if (index == 0) {
		paramtmp->io_port = paramtmp->io_port & ~IOPORT_MSK_FDDALL;
#ifdef USE_IOPORT_FDD
		AG_WidgetEnable(chkIOPort[0]);
		AG_WidgetEnable(chkIOPort[1]);
#endif
	} else if (index == 1) {
		paramtmp->io_port = (paramtmp->io_port & ~IOPORT_MSK_FDDALL) | IOPORT_MSK_3FDD;
#ifdef USE_IOPORT_FDD
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
#endif
	} else if (index == 2 || index == 3) {
		paramtmp->io_port = (paramtmp->io_port & ~IOPORT_MSK_FDDALL) | IOPORT_MSK_5FDD;
#ifdef USE_IOPORT_FDD
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
#endif
	}
}

void AG_CONFIG_DLG::change_io_port(int index, int status)
{
#ifdef USE_IOPORT_FDD
	if (index == 0 && status & (1 << 0)) {
		paramtmp->io_port &= ~(1 << 1);
	} else if (index == 1 && status & (1 << 1)) {
		paramtmp->io_port &= ~(1 << 0);
	}
#endif
	if (index == 5 && status & (1 << 5)) {
		paramtmp->io_port &= ~(1 << 6);
#if defined(_MBS1)
		paramtmp->io_port &= ~(1 << 9);
#endif
	} else if (index == 6 && status & (1 << 6)) {
		paramtmp->io_port &= ~(1 << 5);
#if defined(_MBS1)
	} else if (index == 9 && status & (1 << 9)) {
		paramtmp->io_port &= ~(1 << 5);
		paramtmp->io_port |= (1 << 6);
#endif
	}
}

void AG_CONFIG_DLG::set_label(char *label, const char *str)
{
	strcpy(label,"  ");
	strcat(label, str);
}

void AG_CONFIG_DLG::set_label(char *label, CMsg::Id msg_id)
{
	strcpy(label,"  ");
	strcat(label, gMessages.Get(msg_id));
}

void AG_CONFIG_DLG::Close(AG_Window *win)
{
	AG_ObjectDetach(win);
	emu->update_config();
	gui->PostEtSystemPause(false);
}

int AG_CONFIG_DLG::SetData(AG_Window *win)
{
	param = *paramtmp;

#if defined(_MBS1)
	config.disptmg_skew = param.disptmg_skew - 2;
	config.curdisp_skew = param.curdisp_skew - 2;
#else
	config.disptmg_skew = param.disptmg_skew;
	config.curdisp_skew = param.curdisp_skew;
	config.exram_size_num = param.exram_size_num;
#endif
	config.use_opengl = (uint8_t)param.use_opengl;
	config.gl_filter_type = (uint8_t)param.gl_filter_type;
	config.led_pos = param.led_pos;
	config.capture_type = param.capture_type;

	config.use_power_off = (param.use_power_off != 0);
	config.dipswitch = (param.dipswitch & 0xff);

	emu->set_parami(VM::ParamFddType, param.fdd_type);
#ifndef USE_IOPORT_FDD
	param.io_port &= ~IOPORT_MSK_FDDALL;
	param.io_port |= (param.fdd_type == FDD_TYPE_3FDD ? IOPORT_MSK_3FDD
		: (param.fdd_type == FDD_TYPE_5FDD
#if defined(_MBS1)
		|| param.fdd_type == FDD_TYPE_5HFDD ? IOPORT_MSK_5FDD : 0));
#else
		|| param.fdd_type == FDD_TYPE_8FDD ? IOPORT_MSK_5FDD : 0));
#endif
#endif
	config.mount_fdd = param.mount_fdd;
	config.delay_fdd = (param.delayfd1 ? MSK_DELAY_FDSEARCH : 0)
					| (param.delayfd2 ? MSK_DELAY_FDSEEK : 0);

	emu->set_parami(VM::ParamIOPort, param.io_port);
#if defined(_MBS1)
	emu->set_parami(VM::ParamSysMode, (1 - param.sys_mode_rev) | (config.sys_mode & ~1));
#endif
	int val = 0;

	config.wav_reverse = (param.wav_reverse == 1);
	config.wav_half = (param.wav_half == 1);
	config.wav_correct = (param.wav_correct != 0);
	config.wav_correct_type = (param.wav_correct > 0 ? (uint8_t)(param.wav_correct - 1) : 0);
	if (sscanf(param.wav_correct_amp[0], "%d", &val) && val >= 100 && val <= 5000) {
		config.wav_correct_amp[0] = val;
	}
	if (sscanf(param.wav_correct_amp[1], "%d", &val) && val >= 100 && val <= 5000) {
		config.wav_correct_amp[1] = val;
	}

	config.wav_sample_rate = (uint8_t)param.wav_sample_rate;
	config.wav_sample_bits = (uint8_t)param.wav_sample_bits;

	for(int i=0; i<MAX_PRINTER; i++) {
		config.printer_server_host[i].Set(param.prn_host[i]);
		val = 0;
		if (sscanf(param.prn_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.printer_server_port[i] = val;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		config.comm_server_host[i].Set(param.com_host[i]);
		val = 0;
		if (sscanf(param.com_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.comm_server_port[i] = val;
		}
		config.comm_dipswitch[i] = param.com_dipsw[i];
	}
#ifdef USE_DEBUGGER
	config.debugger_server_host.Set(param.dbgr_host);
	val = 0;
	if (sscanf(param.dbgr_port, "%d", &val) == 1 && val > 0 && val < 65535) {
		config.debugger_server_port = val;
	}
#endif

	config.snapshot_path.Set(param.snap_path);
	config.font_path.Set(param.font_path);
	config.msgboard_msg_fontname.Set(param.msg_font_name);
	config.msgboard_info_fontname.Set(param.info_font_name);
	val = 0;
	if (sscanf(param.msg_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_msg_fontsize = val;
	}
	val = 0;
	if (sscanf(param.info_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_info_fontsize = val;
	}
	config.menu_fontname.Set(param.menu_font_name);
	val = 0;
	if (sscanf(param.menu_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.menu_fontsize = val;
	}

	// language
	clocale->ChooseLocaleName(lang_list, param.lang_selidx, config.language);

	config.rom_path.Set(param.rom_path);

	config.show_dlg = (param.showmsg_undefop ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP);

#if defined(_MBS1)
	emu->set_parami(VM::ParamExMemNum, param.exram_num);
	config.mem_nowait = (param.mem_nowait == 1);
	config.opn_clock = param.fmopn_clock_num;
	config.opn_irq = param.fmopn_irq;
	config.use_opn_expsg = (param.use_opn_expsg == 1);
#if defined(USE_Z80B_CARD)
	config.z80b_card_out_irq = param.z80b_card_out_irq;
#endif
#endif

	// set message font
#ifdef USE_MESSAGE_BOARD
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
#endif
	// set menu font
//	gui->ConfigLoad();
	gui->ChangeLedBox(param.led_show);
	gui->ChangeLedBoxPosition(config.led_pos);
	config.save();
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif

	delete paramtmp;
	paramtmp = NULL;

	return 1;
}

void AG_CONFIG_DLG::show_dir_dlg(AG_Window *win, DirDlgType type)
{
	switch(type) {
	case RomPath:
		dirdlg->CreateLoad(CMSG(ROM_Path), paramtmp->rom_path, sizeof(paramtmp->rom_path), win);
		break;
	case SnapShotPath:
		dirdlg->CreateLoad(CMSG(Snapshot_Path), paramtmp->snap_path, sizeof(paramtmp->snap_path), win);
		break;
	case FontPath:
		dirdlg->CreateLoad(CMSG(Font_Path), paramtmp->font_path, sizeof(paramtmp->font_path), win);
		break;
	}
}

/*
 * Event Handler (static)
 */
void AG_CONFIG_DLG::OnOk(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	if (dlg->SetData(win)) {
		dlg->Close(win);
	}
}

void AG_CONFIG_DLG::OnClose(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->Close(win);
}

void AG_CONFIG_DLG::OnChangeCorrect(AG_Event *event)
{
//	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
//	AG_Radio *rad = (AG_Radio *)AG_PTR(2);
//
//	if (dlg->paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}
}

void AG_CONFIG_DLG::OnChangeSampleRate(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_rate = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeSampleBits(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_bits = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFddType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = (int)AG_INT(2);

	dlg->change_fdd_type(index);
}

void AG_CONFIG_DLG::OnChangeIOPort(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = AG_INT(2);
	int status = AG_INT(3);

	dlg->change_io_port(index, status);
}

void AG_CONFIG_DLG::OnChangeDisptmgSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->disptmg_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCurdispSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->curdisp_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeUseOpenGL(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->use_opengl = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeGLFilterType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->gl_filter_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedShow(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_show = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedPos(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_pos = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCaptureType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->capture_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCommBaud(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int num   = AG_INT(2);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(3);
	int state = AG_INT(4);

	if (state) {
		dlg->paramtmp->com_dipsw[num] = item->label + 1;
	}
}

#if defined(_MBS1)
void AG_CONFIG_DLG::OnChangeExtRam(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->exram_num = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFmOpnClock(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->fmopn_clock_num = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFmOpnIrq(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->fmopn_irq = item->label;
	}
}
#if defined(USE_Z80B_CARD)
void AG_CONFIG_DLG::OnChangeZ80BCardIrq(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->z80b_card_out_irq = item->label;
	}
}
#endif
#endif

void AG_CONFIG_DLG::OnChangeLanguage(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->lang_selidx = item->label;
	}
}

void AG_CONFIG_DLG::OnShowDirDlg(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);
	DirDlgType type = (DirDlgType)AG_INT(3);

	dlg->show_dir_dlg(win, type);
}

}; /* namespace GUI_AGAR */
