/** @file win_configbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.24 -

	@brief [ config box ]
*/
#include "win_configbox.h"
#include "../../emu.h"
#ifdef USE_MESSAGE_BOARD
#include "../../msgboard.h"
#endif
#include "win_folderbox.h"
#include "win_filebox.h"
#include "win_fontbox.h"
#include "../../res/resource.h"
#include "../../config.h"
#include "../../clocale.h"
#include "../../utility.h"
#include "../../rec_video_defs.h"
#include "win_gui.h"

namespace GUI_WIN
{

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

#if defined(_MBS1)
#define CONFIG_TABS 6
#else
#define CONFIG_TABS 5
#endif

static const _TCHAR *rate_list[] = {
	_T("11025"), _T("22050"), _T("44100"), _T("48000"), NULL
};
static const _TCHAR *bits_list[] = {
	_T("8"), _T("16"), NULL
};
static const _TCHAR *skew_list[] = {
#if defined(_MBS1)
	_T("-2"), _T("-1"), _T("0"), _T("1"), _T("2"), NULL
#else
	_T("0"), _T("1"), _T("2"), NULL
#endif
};
#ifdef USE_DIRECT3D
static const CMsg::Id d3d_use_list[] = {
	CMsg::No, CMsg::Yes_sync, CMsg::Yes_async, CMsg::End
};
static const CMsg::Id d3d_filter_list[] = {
	CMsg::None_, CMsg::Point, CMsg::Linear, CMsg::End
};
#endif
#ifdef USE_OPENGL
static const CMsg::Id opengl_use_list[] = {
#ifdef USE_SDL2
	CMsg::Default,
#else
	CMsg::No,
#endif
	CMsg::Yes_sync, CMsg::Yes_async, CMsg::End
};
static const CMsg::Id opengl_filter_list[] = {
	CMsg::Nearest_Neighbour, CMsg::Linear, CMsg::End
};
#endif
static const CMsg::Id led_show_list[] = {
	CMsg::Hide, CMsg::Show_Inside, CMsg::Show_Outside, CMsg::End
};
static const CMsg::Id led_pos_list[] = {
	CMsg::LeftTop, CMsg::RightTop, CMsg::LeftBottom, CMsg::RightBottom, CMsg::End
};
static const _TCHAR *cap_type_list[] = {
	_T("BMP"),
#ifdef USE_CAP_SCREEN_WIN
	_T("PNG"),
#endif
	NULL
};
static const CMsg::Id comm_baud_list[] = {
	CMsg::S_300baud_F_1200baud, CMsg::S_600baud_F_2400baud, CMsg::S_1200baud_F_4800baud, CMsg::S_2400baud_F_9600baud, CMsg::End
};
#if defined(_MBS1)
static const CMsg::Id exram_size_list[] = {
	CMsg::None_, CMsg::S64KB, CMsg::S128KB, CMsg::S256KB, CMsg::S512KB, CMsg::End
};
static const _TCHAR *fmopn_clock_list[] = {
	_T("3,579,545"), _T("1,008,000"), _T("2,016,000"), _T("4,032,000"), NULL
};
static const CMsg::Id fmopn_irq_list[] = {
	CMsg::None_, CMsg::IRQ, CMsg::FIRQ, CMsg::End
};
#if defined(USE_Z80B_CARD)
static const CMsg::Id z80bcard_irq_list[] = {
	CMsg::IRQ, CMsg::NMI, CMsg::End
};
#endif
#endif

ConfigBox::ConfigBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_CONFIGURE, new_font, new_emu, new_gui)
{
	hInstance = hInst;

	fdd_type  = config.fdd_type;
	io_port = config.io_port;

	selected_tabctrl = 0;
}

ConfigBox::~ConfigBox()
{
}

INT_PTR ConfigBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hCtrl;

	//
	CDialogBox::onInitDialog(message, wParam, lParam);

	fdd_type = emu->get_parami(VM::ParamFddType);
	io_port = emu->get_parami(VM::ParamIOPort);
#if defined(_MBS1)
	sys_mode = emu->get_parami(VM::ParamSysMode);
	exram_size_num = emu->get_parami(VM::ParamExMemNum);
#endif

	// power off
	CheckDlgButton(hDlg, IDC_CHK_POWEROFF, config.use_power_off);

	// MODE switch
#if defined(_MBS1)
	CheckDlgButton(hDlg, ID_DIPSWITCH1, (sys_mode & 1) != 0);
	CheckDlgButton(hDlg, ID_DIPSWITCH2, (sys_mode & 1) == 0);
#endif
	CheckDlgButton(hDlg, ID_DIPSWITCH3, (config.dipswitch & 4) != 0);

	// FDD type select
	CheckDlgButton(hDlg, IDC_RADIO_NOFDD, fdd_type == FDD_TYPE_NOFDD);
	CheckDlgButton(hDlg, IDC_RADIO_3FDD, fdd_type == FDD_TYPE_3FDD);
	CheckDlgButton(hDlg, IDC_RADIO_5FDD, fdd_type == FDD_TYPE_5FDD);
#if defined(_MBS1)
	CheckDlgButton(hDlg, IDC_RADIO_5FDD1M, fdd_type == FDD_TYPE_5HFDD);
#else
	CheckDlgButton(hDlg, IDC_RADIO_8FDD, fdd_type == FDD_TYPE_8FDD);
#endif
	for(int i=0; i<MAX_DRIVE; i++) {
		CheckDlgButton(hDlg, IDC_CHK_FDD_MOUNT0 + i, (config.mount_fdd & (1 << i)) != 0);
	}
	CheckDlgButton(hDlg, IDC_CHK_DELAYFD1, FLG_DELAY_FDSEARCH);
	CheckDlgButton(hDlg, IDC_CHK_DELAYFD2, FLG_DELAY_FDSEEK);

	// I/O port address
	for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if ((1 << i) & IOPORT_MSK_ALL) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT1 + i, (io_port & (1 << i)) != 0);
		}
	}

	// Load wave file
	CheckDlgButton(hDlg, IDC_CHK_REVERSE, config.wav_reverse);
	CheckDlgButton(hDlg, IDC_CHK_HALFWAVE, config.wav_half);

	CheckDlgButton(hDlg, IDC_RADIO_NOCORR, !config.wav_correct);
	CheckDlgButton(hDlg, IDC_RADIO_COSW, config.wav_correct && config.wav_correct_type == 0);
	CheckDlgButton(hDlg, IDC_RADIO_SINW, config.wav_correct && config.wav_correct_type == 1);

	SetDlgItemInt(hDlg, IDC_TXT_CORRAMP0, config.wav_correct_amp[0], false);
	SetDlgItemInt(hDlg, IDC_TXT_CORRAMP1, config.wav_correct_amp[1], false);
//	if (config.wav_correct) {
//		CheckDlgButton(hDlg, IDC_CHK_CORRECT, 1);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//		EnableWindow(hCtrl, true);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//		EnableWindow(hCtrl, true);
//	} else {
//		CheckDlgButton(hDlg, IDC_CHK_CORRECT, 0);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//		EnableWindow(hCtrl, false);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//		EnableWindow(hCtrl, false);
//	}

	// Save wave file
	SetComboBoxItems(IDC_COMBO_SRATE, rate_list, config.wav_sample_rate);
	SetComboBoxItems(IDC_COMBO_SBITS, bits_list, config.wav_sample_bits);

	// crtc
#if defined(_MBS1)
	SetComboBoxItems(IDC_COMBO_DISPTMG, skew_list, config.disptmg_skew + 2);
	SetComboBoxItems(IDC_COMBO_CURDISP, skew_list, config.curdisp_skew + 2);
#else
	SetComboBoxItems(IDC_COMBO_DISPTMG, skew_list, config.disptmg_skew);
	SetComboBoxItems(IDC_COMBO_CURDISP, skew_list, config.curdisp_skew);
#endif

#ifdef USE_DIRECT3D
	// d3d use
	SetComboBoxItems(IDC_COMBO_D3D_USE, (CMsg::Id *)&d3d_use_list, config.use_direct3d, true);

	// d3d filter type
	SetComboBoxItems(IDC_COMBO_D3D_FILTER, (CMsg::Id *)&d3d_filter_list, config.d3d_filter_type, true);
#endif
#ifdef USE_OPENGL
	// opengl use
	SetComboBoxItems(IDC_COMBO_OPENGL_USE, opengl_use_list, config.use_opengl, true);

	// opengl filter type
	SetComboBoxItems(IDC_COMBO_OPENGL_FILTER, opengl_filter_list, config.gl_filter_type, true);
#endif

	// led show
	int led_show = gui->GetLedBoxPhase(-1);
	SetComboBoxItems(IDC_COMBO_LED_SHOW, (CMsg::Id *)&led_show_list, led_show, true);
	// led pos
	SetComboBoxItems(IDC_COMBO_LED_POS, (CMsg::Id *)&led_pos_list, config.led_pos, true);

	// capture type
	SetComboBoxItems(IDC_COMBO_CAPTURE_TYPE, cap_type_list, config.capture_type, true);

	// comm baud
	for(int j=0; j<MAX_COMM; j++) {
		SetComboBoxItems(IDC_COMBO_COM0 + j, (CMsg::Id *)&comm_baud_list, config.comm_dipswitch[j] - 1, true);
	}

	// snapshot path
	SetDlgItemText(hDlg, IDC_SNAP_PATH, config.snapshot_path.GetM());
	// font file
	SetDlgItemText(hDlg, IDC_FONT_FILE, config.font_path.GetM());
	// message font name
	SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, config.msgboard_info_fontname.GetM());
	SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, config.msgboard_msg_fontname.GetM());
	// message font size
	SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, config.msgboard_info_fontsize, FALSE);
	SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, config.msgboard_msg_fontsize, FALSE);

	// hostname, port
	for(int i=0; i<MAX_PRINTER; i++) {
		SetDlgItemText(hDlg, IDC_HOSTNAME_LPT0 + i, config.printer_server_host[i]);
		SetDlgItemInt(hDlg, IDC_PORT_LPT0 + i, config.printer_server_port[i], TRUE);
	}
	for(int i=0; i<MAX_COMM; i++) {
		SetDlgItemText(hDlg, IDC_HOSTNAME_COM0 + i, config.comm_server_host[i]);
		SetDlgItemInt(hDlg, IDC_PORT_COM0 + i, config.comm_server_port[i], TRUE);
	}
#ifdef USE_DEBUGGER
	SetDlgItemText(hDlg, IDC_HOSTNAME_DBGR, config.debugger_server_host);
	SetDlgItemInt(hDlg, IDC_PORT_DBGR, config.debugger_server_port, TRUE);
#endif

	// rom path
	SetDlgItemText(hDlg, IDC_ROM_PATH, config.rom_path.GetM());
#if defined(_MBS1)
	// exram size
	SetComboBoxItems(IDC_COMBO_EXMEM, (const CMsg::Id *)exram_size_list, exram_size_num, true);
	if (0 <= config.exram_size_num && config.exram_size_num <= 4) {
		_TCHAR str[100];
		UTILITY::tcscpy(str, 100, CMSGM(LB_Now_SP));
		UTILITY::tcscat(str, 100, CMSGVM(exram_size_list[config.exram_size_num]));
		UTILITY::tcscat(str, 100, _T(")"));
		SetDlgItemText(hDlg, IDC_STATIC_EXMEM_NOW, str);
	}
	// no wait
	CheckDlgButton(hDlg, IDC_CHK_MEMNOWAIT, config.mem_nowait);
#else
	// extended memory
	CheckDlgButton(hDlg, IDC_CHK_EXMEM, config.exram_size_num == 1);
#endif
	// undef opcode
	CheckDlgButton(hDlg, IDC_CHK_UNDEFOP, FLG_SHOWMSG_UNDEFOP != 0);
#if defined(_MBS1) && defined(USE_Z80B_CARD)
	// z80bcard irq
	SetComboBoxItems(IDC_COMBO_Z80BCARD_IRQ, (const CMsg::Id *)z80bcard_irq_list, config.z80b_card_out_irq, true);
#endif

#if defined(_MBS1)
	// fmopn clock
	SetComboBoxItems(IDC_COMBO_FMOPN, fmopn_clock_list, config.opn_clock, false);
	// fmopn irq
	SetComboBoxItems(IDC_COMBO_FMOPN_IRQ, (const CMsg::Id *)fmopn_irq_list, config.opn_irq, true);
	// use opn instead of psg
	CheckDlgButton(hDlg, IDC_CHK_USE_OPN_EXPSG, config.use_opn_expsg);
#endif

	// get language
	lang_list.Clear();
	clocale->GetLocaleNamesWithDefault(lang_list);
	int lang_selidx = clocale->SelectLocaleNameIndex(lang_list, config.language);
	SetComboBoxItems(IDC_COMBO_LANGUAGE, lang_list, lang_selidx);

	// layout
	// tab control
	CBox *box_all = new CBox(CBox::VerticalBox, 0, margin, _T("all"));
	CBox *box_tab = AdjustTabControl(box_all, IDC_TAB1, IDC_STATIC_0);

	HWND hTabCtrl = GetDlgItem(hDlg, IDC_TAB1);

	TCITEM tcitm;
	tcitm.mask = TCIF_TEXT;

	tcitm.pszText = (LPSTR)CMSGM(Mode);
	TabCtrl_InsertItem(hTabCtrl , 0 , &tcitm);
	tcitm.pszText = (LPSTR)CMSGM(Screen);
	TabCtrl_InsertItem(hTabCtrl , 1 , &tcitm);
	tcitm.pszText = (LPSTR)CMSGM(Tape_FDD);
	TabCtrl_InsertItem(hTabCtrl , 2 , &tcitm);
	tcitm.pszText = (LPSTR)CMSGM(Network);
	TabCtrl_InsertItem(hTabCtrl , 3 , &tcitm);
	tcitm.pszText = (LPSTR)CMSGM(CPU_Memory);
	TabCtrl_InsertItem(hTabCtrl , 4 , &tcitm);
#if defined(_MBS1)
	tcitm.pszText = (LPSTR)CMSGM(Sound);
	TabCtrl_InsertItem(hTabCtrl , 5 , &tcitm);
#endif
	TabCtrl_SetCurSel(hTabCtrl, selected_tabctrl);

	// 0:Mode
	CBox *box_0all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("0all"));

	CBox *box_0h = box_0all->AddBox(CBox::HorizontalBox, 0, 0, _T("0h"));

	CBox *box_0lv = box_0h->AddBox(CBox::VerticalBox, 0, 0, _T("0lv"));

	// DIP switch
#if defined(_MBS1)
	CBox *box_sysmode_sw = AdjustGroup(box_0lv, IDC_STATIC_1, CBox::VerticalBox);

	for(int id=ID_DIPSWITCH1; id<=ID_DIPSWITCH2; id++) {
		CBox *box_mode_sw = box_sysmode_sw->AddBox(CBox::HorizontalBox);
		AdjustStatic(box_mode_sw, IDC_STATIC_MODE_SW);
		AdjustCheckBox(box_mode_sw, id, 160, 0);
	}
	CBox *box_modec_sw = box_sysmode_sw->AddBox(CBox::HorizontalBox, 0, 0, _T("modec_sw"));
	box_modec_sw->AddSpace(20, 10);
	AdjustCheckBox(box_modec_sw, ID_DIPSWITCH3);
#else
	CBox *box_dip_sw = AdjustGroup(box_0lv, IDC_STATIC_1, CBox::VerticalBox);
	CBox *box_mode_sw = box_dip_sw->AddBox(CBox::HorizontalBox, 0, 0, _T("mode_sw"));
	AdjustStatic(box_mode_sw, IDC_STATIC_MODE_SW);
	AdjustCheckBox(box_mode_sw, ID_DIPSWITCH3);
#endif

	// FDD type select
	CBox *box_fdd_type = AdjustGroup(box_0lv, IDC_STATIC_2, CBox::VerticalBox);
	for(int id=IDC_RADIO_NOFDD;
#if defined(_MBS1)
			id<=IDC_RADIO_5FDD1M;
#else
			id<=IDC_RADIO_8FDD;
#endif
			id++) {
		CBox *box_fdd_type0 = box_fdd_type->AddBox(CBox::HorizontalBox);
		AdjustStatic(box_fdd_type0, IDC_STATIC_ALLOW);
		AdjustCheckBox(box_fdd_type0, id, 160, 0);
	}

	AdjustCheckBox(box_0lv, IDC_CHK_POWEROFF);

	// I/O port address
	CBox *box_0rv = box_0h->AddBox(CBox::VerticalBox, 0, 0, _T("0rv"));
	CBox *box_ioport = AdjustGroup(box_0rv, IDC_STATIC_3, CBox::VerticalBox);

	for(int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if ((1 << i) & IOPORT_MSK_ALL) {
			CBox *box_ioport0 = box_ioport->AddBox(CBox::HorizontalBox);
			AdjustStatic(box_ioport0, IDC_STATIC_IOPORT1 + i);
			AdjustCheckBox(box_ioport0, IDC_CHK_IOPORT1 + i, 200, 0);
		}
	}

	// text
	AdjustStatic(box_0all, IDC_STATIC_RESTART);


	// 1:Screen
	CBox *box_1all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("1all"));

	CBox *box_1h = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("1h"));

	CBox *box_1lv = box_1h->AddBox(CBox::VerticalBox, 0, 0, _T("1lv"));

	// DirectX
	CBox *box_d3d = AdjustGroup(box_1lv, IDC_STATIC_7, CBox::HorizontalBox);

	CBox *box_d3d_tit = box_d3d->AddBox(CBox::VerticalBox, 0, 0, _T("d3d_tit"));
#ifdef USE_DIRECT3D
	AdjustStatic(box_d3d_tit, IDC_STATIC_D3D_USE);
#endif
#ifdef USE_OPENGL
	AdjustStatic(box_d3d_tit, IDC_STATIC_OPENGL_USE);
#endif
#ifdef USE_DIRECT3D
	AdjustStatic(box_d3d_tit, IDC_STATIC_D3D_FILTER);
#endif
#ifdef USE_OPENGL
	AdjustStatic(box_d3d_tit, IDC_STATIC_OPENGL_FILTER);
#endif

	CBox *box_d3d_com = box_d3d->AddBox(CBox::VerticalBox, 0, 0, _T("d3d_com"));
#ifdef USE_DIRECT3D
	AdjustComboBox(box_d3d_com, IDC_COMBO_D3D_USE, 6);
#endif
#ifdef USE_OPENGL
	AdjustComboBox(box_d3d_com, IDC_COMBO_OPENGL_USE, 6);
#endif
#ifdef USE_DIRECT3D
	AdjustComboBox(box_d3d_com, IDC_COMBO_D3D_FILTER, 6);
#endif
#ifdef USE_OPENGL
	AdjustComboBox(box_d3d_com, IDC_COMBO_OPENGL_FILTER, 6);
#endif

	// crtc
	CBox *box_1rv = box_1h->AddBox(CBox::VerticalBox, 0, 0, _T("1rv"));
//	(box_1rv);

	CBox *box_crtc = AdjustGroup(box_1rv, IDC_STATIC_6, CBox::HorizontalBox);

	CBox *box_crtc_tit = box_crtc->AddBox(CBox::VerticalBox, 0, 0, _T("crtc_tit"));
	AdjustStatic(box_crtc_tit, IDC_STATIC_DISPTMG);
	AdjustStatic(box_crtc_tit, IDC_STATIC_CURDISP);

	CBox *box_crtc_com = box_crtc->AddBox(CBox::VerticalBox, 0, 0, _T("crtc_com"));
	AdjustComboBox(box_crtc_com, IDC_COMBO_DISPTMG, 2);
	AdjustComboBox(box_crtc_com, IDC_COMBO_CURDISP, 2);

	// LED
	CBox *box_led = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("led"));
	AdjustStatic(box_led, IDC_STATIC_LED_SHOW);
	AdjustComboBox(box_led, IDC_COMBO_LED_SHOW, 8);
	AdjustStatic(box_led, IDC_STATIC_LED_POS);
	AdjustComboBox(box_led, IDC_COMBO_LED_POS, 8);

	// Capture Type
	CBox *box_capt = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("capture"));
	AdjustStatic(box_capt, IDC_STATIC_CAPTURE_TYPE);
	AdjustComboBox(box_capt, IDC_COMBO_CAPTURE_TYPE, 8);

	// Snapshot path , Font file , Message font
	CBox *box_snapshot = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("snapshot"));
	AdjustStatic(box_snapshot, IDC_STATIC_SNAP_PATH, 100);
	CBox *box_fontfile = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("fontfile"));
	AdjustStatic(box_fontfile, IDC_STATIC_FONT_FILE, 100);
	CBox *box_fontns = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("fontns"));
	AdjustStatic(box_fontns, IDC_STATIC_MSG_FONT_NAME_S, 100);
	CBox *box_fontnl = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("fontnl"));
	AdjustStatic(box_fontnl, IDC_STATIC_MSG_FONT_NAME_L, 100);

	//
	AdjustEditBox(box_snapshot, IDC_SNAP_PATH, 30);
	AdjustButton(box_snapshot, IDC_BTN_SNAP_PATH, 5);

	AdjustEditBox(box_fontfile, IDC_FONT_FILE, 30);
	AdjustButton(box_fontfile, IDC_BTN_FONT_FILE, 5);

	AdjustEditBox(box_fontns, IDC_MSG_FONT_NAME_S, 10);
	AdjustStatic(box_fontns, IDC_STATIC_MSG_FONT_SIZE_S);
	AdjustEditBox(box_fontns, IDC_MSG_FONT_SIZE_S, 3);
	AdjustButton(box_fontns, IDC_BTN_FONT_NAME_S, 5);

	AdjustEditBox(box_fontnl, IDC_MSG_FONT_NAME_L, 10);
	AdjustStatic(box_fontnl, IDC_STATIC_MSG_FONT_SIZE_L);
	AdjustEditBox(box_fontnl, IDC_MSG_FONT_SIZE_L, 3);
	AdjustButton(box_fontnl, IDC_BTN_FONT_NAME_L, 5);

	CBox *box_lang = box_1all->AddBox(CBox::HorizontalBox, 0, 0, _T("lang"));
	AdjustStatic(box_lang, IDC_STATIC_LANGUAGE);
	AdjustComboBox(box_lang, IDC_COMBO_LANGUAGE, 8);

	// text
	AdjustStatic(box_1all, IDC_STATIC_RESTART_2);

	// 2:Tape, FDD
	CBox *box_2all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("2all"));

	CBox *box_2h = box_2all->AddBox(CBox::HorizontalBox, 0, 0, _T("2h"));

	CBox *box_2lv = box_2h->AddBox(CBox::VerticalBox, 0, 0, _T("2lv"));

	// Load wav file
	CBox *box_ldwav = AdjustGroup(box_2lv, IDC_STATIC_4, CBox::VerticalBox);

	CBox *box_ldwav0 = box_ldwav->AddBox(CBox::HorizontalBox, 0, 0, _T("ldwav0"));
	AdjustCheckBox(box_ldwav0, IDC_CHK_REVERSE);
	AdjustCheckBox(box_ldwav0, IDC_CHK_HALFWAVE);
	CBox *box_ldwav1 = box_ldwav->AddBox(CBox::HorizontalBox, 0, 0, _T("ldwav1"));
	AdjustStatic(box_ldwav1, IDC_STATIC_CORRECT);
	AdjustCheckBox(box_ldwav1, IDC_RADIO_NOCORR);
	AdjustCheckBox(box_ldwav1, IDC_RADIO_COSW);
	AdjustCheckBox(box_ldwav1, IDC_RADIO_SINW);
	CBox *box_ldwav2 = box_ldwav->AddBox(CBox::HorizontalBox, 0, 0, _T("ldwav2"));
	AdjustStatic(box_ldwav2, IDC_STATIC_CORRAMP0);
	AdjustEditBox(box_ldwav2, IDC_TXT_CORRAMP0, 4);
	AdjustStatic(box_ldwav2, IDC_STATIC_CORRAMP1);
	AdjustEditBox(box_ldwav2, IDC_TXT_CORRAMP1, 4);

	// Save wav file
	CBox *box_2rv = box_2h->AddBox(CBox::VerticalBox, 0, 0, _T("2rv"));

	CBox *box_svwav = AdjustGroup(box_2rv, IDC_STATIC_5, CBox::HorizontalBox);

	CBox *box_svwav_t = box_svwav->AddBox(CBox::VerticalBox, 0, 0, _T("svwav_t"));
	AdjustStatic(box_svwav_t, IDC_STATIC_SRATE);
	AdjustStatic(box_svwav_t, IDC_STATIC_SBITS);

	CBox *box_svwav_c = box_svwav->AddBox(CBox::VerticalBox, 0, 0, _T("svwav_c"));
	AdjustComboBox(box_svwav_c, IDC_COMBO_SRATE, 4);
	AdjustComboBox(box_svwav_c, IDC_COMBO_SBITS, 4);

	// FDD
	CBox *box_fdd = AdjustGroup(box_2all, IDC_STATIC_8, CBox::VerticalBox);

	// mount fdd
	CBox *box_fdd_mount = box_fdd->AddBox(CBox::HorizontalBox, 0, 0, _T("fdd_mount"));
	AdjustStatic(box_fdd_mount, IDC_STATIC_FDD_MOUNT);
	for(int i=0; i<MAX_DRIVE; i++) {
		AdjustCheckBox(box_fdd_mount, IDC_CHK_FDD_MOUNT0 + i);
	}

	AdjustCheckBox(box_fdd, IDC_CHK_DELAYFD1);
	AdjustCheckBox(box_fdd, IDC_CHK_DELAYFD2);

	// 3:Network
	CBox *box_3all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("3all"));

	CBox *box_3h = box_3all->AddBox(CBox::HorizontalBox, 0, 0, _T("3h"));

	// LPT0 .. COM2
	CBox *box_3tit = box_3h->AddBox(CBox::VerticalBox, 0, 0, _T("3tit"));
	for(int i=0; i<6; i++) {
		AdjustStatic(box_3tit, IDC_STATIC_NET0 + i);
	}
#ifdef USE_DEBUGGER
	CBox *box_3dbg = box_3all->AddBox(CBox::HorizontalBox, 0, 0, _T("3dbg"));
	AdjustStatic(box_3dbg, IDC_STATIC_DBGR);
#endif
	CBox *box_3nam = box_3h->AddBox(CBox::VerticalBox, 0, 0, _T("3nam"));
	for(int i=0; i<6; i++) {
		if (i == 0) {
			AdjustStatic(box_3nam, IDC_HOSTNAME_NET0 + i);
		} else {
			AdjustEditBox(box_3nam, IDC_HOSTNAME_NET0 + i, 12);
		}
	}
#ifdef USE_DEBUGGER
	AdjustEditBox(box_3dbg, IDC_HOSTNAME_DBGR, 12);
#endif
	CBox *box_3pot = box_3h->AddBox(CBox::VerticalBox, 0, 0, _T("3pot"));
	for(int i=0; i<6; i++) {
		if (i == 0) {
			AdjustStatic(box_3pot, IDC_PORT_NET0 + i);
		} else if (i >= 4) {
			CBox *box_3pot0 = box_3pot->AddBox(CBox::HorizontalBox, 0, 0, _T("3pot0"));
			AdjustEditBox(box_3pot0, IDC_PORT_NET0 + i, 6);
			AdjustComboBox(box_3pot0, IDC_COMBO_COM0 + i - 4, 10);
		} else {
			AdjustEditBox(box_3pot, IDC_PORT_NET0 + i, 6);
		}
	}
#ifdef USE_DEBUGGER
	AdjustEditBox(box_3dbg, IDC_PORT_DBGR, 6);
#endif

	// 4:CPU, Memory
	CBox *box_4all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("4all"));

	CBox *box_rompath = box_4all->AddBox(CBox::HorizontalBox, 0, 0, _T("rompath"));
	AdjustStatic(box_rompath, IDC_STATIC_ROM_PATH);
	AdjustEditBox(box_rompath, IDC_ROM_PATH, 30);
	AdjustButton(box_rompath, IDC_BTN_ROM_PATH, 5);

#if defined(_MBS1)
	// exram size
	CBox *box_exram = box_4all->AddBox(CBox::HorizontalBox, 0, 0, _T("exram"));
	AdjustStatic(box_exram, IDC_STATIC_EXMEM);
	AdjustComboBox(box_exram, IDC_COMBO_EXMEM, 5);
	AdjustStatic(box_exram, IDC_STATIC_EXMEM_NOW);

	// no wait
	AdjustCheckBox(box_4all, IDC_CHK_MEMNOWAIT);
#else
	// ex memory
	AdjustCheckBox(box_4all, IDC_CHK_EXMEM);
#endif

	// undef opcode
	AdjustCheckBox(box_4all, IDC_CHK_UNDEFOP);

#if defined(_MBS1) && defined(USE_Z80B_CARD)
	// z80b card
	CBox *box_z80bint = box_4all->AddBox(CBox::HorizontalBox, 0, 0, _T("z80int"));
	AdjustStatic(box_z80bint, IDC_STATIC_Z80BCARD_IRQ);
	AdjustComboBox(box_z80bint, IDC_COMBO_Z80BCARD_IRQ, 5);
#endif

	// text
	AdjustStatic(box_4all, IDC_STATIC_RESTART_3);

#if defined(_MBS1)
	// 5:Sound
	CBox *box_5all = box_tab->AddBox(CBox::VerticalBox, 0, 0, _T("5all"));

	CBox *box_fmopn = box_5all->AddBox(CBox::HorizontalBox, 0, 0, _T("fmopn"));
	AdjustStatic(box_fmopn, IDC_STATIC_FMOPN);
	AdjustComboBox(box_fmopn, IDC_COMBO_FMOPN, 5);
	AdjustStatic(box_fmopn, IDC_STATIC_FMOPN_2);

	// interrupt
	CBox *box_fmint = box_5all->AddBox(CBox::HorizontalBox, 0, 0, _T("fmint"));
	AdjustStatic(box_fmint, IDC_STATIC_FMOPN_IRQ);
	AdjustComboBox(box_fmint, IDC_COMBO_FMOPN_IRQ, 5);

	// usg opn instead of psg
	AdjustCheckBox(box_5all, IDC_CHK_USE_OPN_EXPSG);

	// text
	AdjustStatic(box_5all, IDC_STATIC_RESTART_4);
#endif

	// button
	CBox *box_btn = box_all->AddBox(CBox::HorizontalBox, CBox::RightPos);
	AdjustButton(box_btn, IDOK, 8);
	AdjustButton(box_btn, IDCANCEL, 8);

	RECT prc;
	GetClientRect(hTabCtrl, &prc);
	TabCtrl_AdjustRect(hTabCtrl, FALSE, &prc);
	box_tab->SetTopMargin(prc.top + 8);

	box_all->Realize(*this);

	int ax, ay;
#if defined(_MBS1)
	hCtrl = GetDlgItem(hDlg, IDC_STATIC_MODE_SW);
	box_sysmode_sw->GetPositionByItem((1 - (config.sys_mode & 1)), ax, ay);
	SetWindowPos(hCtrl, 0, ax, ay + padding, 1, 1, SWP_NOSIZE | SWP_NOZORDER);
#endif
	hCtrl = GetDlgItem(hDlg, IDC_STATIC_ALLOW);
	box_fdd_type->GetPositionByItem(config.fdd_type, ax, ay);
	SetWindowPos(hCtrl, 0, ax, ay + padding, 1, 1, SWP_NOSIZE | SWP_NOZORDER);

	select_tabctrl(selected_tabctrl);

	delete box_all;

	return (INT_PTR)TRUE;
}

INT_PTR ConfigBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef USE_IOPORT_FDD
	HWND hCtrl;
#endif
	WORD wId = LOWORD(wParam);

	if (wId == IDOK) {
		onOK(message, wParam, lParam);
	}
	else if (wId == IDC_RADIO_NOFDD) {
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
#if defined(_MBS1)
		CheckDlgButton(hDlg, IDC_RADIO_5FDD1M, 0);
#else
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#endif
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, true);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, true);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_3FDD) {
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
#if defined(_MBS1)
		CheckDlgButton(hDlg, IDC_RADIO_5FDD1M, 0);
#else
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#endif
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_5FDD) {
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
#if defined(_MBS1)
		CheckDlgButton(hDlg, IDC_RADIO_5FDD1M, 0);
#else
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#endif
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
#if defined(_MBS1)
	else if (wId == IDC_RADIO_5FDD1M) {
#else
	else if (wId == IDC_RADIO_8FDD) {
#endif
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_NOCORR)
	{
		CheckDlgButton(hDlg, IDC_RADIO_COSW, 0);
		CheckDlgButton(hDlg, IDC_RADIO_SINW, 0);
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_COSW)
	{
		CheckDlgButton(hDlg, IDC_RADIO_NOCORR, 0);
		CheckDlgButton(hDlg, IDC_RADIO_SINW, 0);
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_SINW)
	{
		CheckDlgButton(hDlg, IDC_RADIO_NOCORR, 0);
		CheckDlgButton(hDlg, IDC_RADIO_COSW, 0);
		return (INT_PTR)FALSE;
	}
//	else if (wId == IDC_CHK_CORRECT)
//	{
//		if (IsDlgButtonChecked(hDlg, IDC_CHK_CORRECT)) {
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//			EnableWindow(hCtrl, true);
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//			EnableWindow(hCtrl, true);
//		} else {
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//			EnableWindow(hCtrl, false);
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//			EnableWindow(hCtrl, false);
//		}
//		return (INT_PTR)FALSE;
//	}
#ifdef USE_IOPORT_FDD
	else if (wId == IDC_CHK_IOPORT1)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT1)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_CHK_IOPORT2)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT2)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		}
		return (INT_PTR)FALSE;
	}
#endif
	else if (wId == IDC_CHK_IOPORT6)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT6)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT7, 0);
#if defined(_MBS1)
			CheckDlgButton(hDlg, IDC_CHK_IOPORT10, 0);
#endif
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_CHK_IOPORT7)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT7)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT6, 0);
		}
		return (INT_PTR)FALSE;
	}
#if defined(_MBS1)
	else if (wId == IDC_CHK_IOPORT10)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT10)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT6, 0);
			CheckDlgButton(hDlg, IDC_CHK_IOPORT7, 1);
		}
		return (INT_PTR)FALSE;
	}
#endif
	else if (wId == IDC_BTN_SNAP_PATH)
	{
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_SNAP_PATH, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) {
			const _TCHAR *tbuf = emu->application_path();
#if defined(USE_UTF8_ON_MBCS)
			UTILITY::conv_to_native_path(tbuf, buf, _MAX_PATH);
#else
			UTILITY::tcscpy(buf, _MAX_PATH, tbuf);
#endif
		}
		if (fbox.Show(CMSGM(Select_a_folder_to_save_snapshot_images), buf, _MAX_PATH)) {
			SetDlgItemText(hDlg, IDC_SNAP_PATH, buf);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_FILE)
	{
#ifdef USE_WIN
		const CMsg::Id filter[] =
			{ CMsg::Supported_Files_ttf_otf, CMsg::All_Files_, CMsg::End };
		_TCHAR buf[_MAX_PATH];
		FileBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
		if (fbox.Show(
			filter,
			CMSGM(Select_a_font_file_for_showing_messages),
			NULL,
			_T("ttf"),
			false,
			buf)) {
			SetDlgItemText(hDlg, IDC_FONT_FILE, buf);
		}
#endif
#if defined(USE_SDL) || defined(USE_SDL2)
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) {
			const _TCHAR *tbuf = emu->application_path();
#if defined(USE_UTF8_ON_MBCS)
			UTILITY::conv_to_native_path(tbuf, buf, _MAX_PATH);
#else
			UTILITY::tcscpy(buf, _MAX_PATH, tbuf);
#endif
		}
		if (fbox.Show(CMSGM(Select_a_font_folder_for_showing_messages), buf, _MAX_PATH)) {
			SetDlgItemText(hDlg, IDC_FONT_FILE, buf);
		}
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_NAME_L)
	{
		_TCHAR buf[_MAX_PATH];
		double font_size;
		BOOL rc;
		FontBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf, _MAX_PATH);
		font_size = (double)GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, &rc, TRUE);
		if (fbox.Show(
			CMSGM(Select_a_font),
			NULL, buf, _MAX_PATH, &font_size)) {
			SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf);
			SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, (UINT)font_size, TRUE);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_NAME_S)
	{
		_TCHAR buf[_MAX_PATH];
		double font_size;
		BOOL rc;
		FontBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf, _MAX_PATH);
		font_size = (double)GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, &rc, TRUE);
		if (fbox.Show(
			CMSGM(Select_a_font),
			NULL, buf, _MAX_PATH, &font_size)) {
			SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf);
			SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, (UINT)font_size, TRUE);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_ROM_PATH)
	{
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_ROM_PATH, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) {
			const _TCHAR *tbuf = emu->application_path();
#if defined(USE_UTF8_ON_MBCS)
			UTILITY::conv_to_native_path(tbuf, buf, _MAX_PATH);
#else
			UTILITY::tcscpy(buf, _MAX_PATH, tbuf);
#endif
		}
		if (fbox.Show(CMSGM(Select_a_folder_containing_the_rom_images), buf, _MAX_PATH)) {
			SetDlgItemText(hDlg, IDC_ROM_PATH, buf);
		}
		return (INT_PTR)FALSE;
	}

	if (wId == IDOK || wId == IDCANCEL) {
		::EndDialog(hDlg, wId);
		return (INT_PTR)TRUE;
	}
	return (INT_PTR)FALSE;
}

INT_PTR ConfigBox::onNotify(UINT message, WPARAM wParam, LPARAM lParam)
{
	// change tab
	LPNMHDR lpNmHdr = (NMHDR *)lParam;
	int i;
	if (lpNmHdr->idFrom == IDC_TAB1) {
		switch (lpNmHdr->code) {
		case TCN_SELCHANGE:
			i = TabCtrl_GetCurSel(lpNmHdr->hwndFrom);
			select_tabctrl(i);
			break;
		}
	}
	return (INT_PTR)TRUE;
}

INT_PTR ConfigBox::onOK(UINT message, WPARAM wParam, LPARAM lParam)
{
	_TCHAR buf[_MAX_PATH];
	int valuel;
	BOOL rc;

	// power off
	config.use_power_off = (IsDlgButtonChecked(hDlg, IDC_CHK_POWEROFF) == BST_CHECKED);

	// MODE switch
#if defined(_MBS1)
	if (IsDlgButtonChecked(hDlg, ID_DIPSWITCH1)) {
		sys_mode |= 1;
	} else {
		sys_mode &= ~1;
	}
#endif
	if (IsDlgButtonChecked(hDlg, ID_DIPSWITCH3)) {
		config.dipswitch |= 4;
	} else {
		config.dipswitch &= ~4;
	}

	// fdd type
	fdd_type = (IsDlgButtonChecked(hDlg, IDC_RADIO_3FDD) ? FDD_TYPE_3FDD : 0)
			 + (IsDlgButtonChecked(hDlg, IDC_RADIO_5FDD) ? FDD_TYPE_5FDD : 0)
#if defined(_MBS1)
			 + (IsDlgButtonChecked(hDlg, IDC_RADIO_5FDD1M) ? FDD_TYPE_5HFDD : 0);
#else
			 + (IsDlgButtonChecked(hDlg, IDC_RADIO_8FDD) ? FDD_TYPE_8FDD : 0);
#endif
#ifndef USE_IOPORT_FDD
	io_port &= ~IOPORT_MSK_FDDALL;
	io_port |= (fdd_type == FDD_TYPE_3FDD ? IOPORT_MSK_3FDD
			 : (fdd_type == FDD_TYPE_5FDD
#if defined(_MBS1)
			 || fdd_type == FDD_TYPE_5HFDD ? IOPORT_MSK_5FDD : 0));
#else
			 || fdd_type == FDD_TYPE_8FDD ? IOPORT_MSK_5FDD : 0));
#endif
#endif

	config.mount_fdd = 0;
	for (int i=0; i<MAX_DRIVE; i++) {
		config.mount_fdd |= (IsDlgButtonChecked(hDlg, IDC_CHK_FDD_MOUNT0 + i) == BST_CHECKED ? 1 << i : 0);
	}
	config.delay_fdd = (IsDlgButtonChecked(hDlg, IDC_CHK_DELAYFD1) == BST_CHECKED ? MSK_DELAY_FDSEARCH : 0)
					| (IsDlgButtonChecked(hDlg, IDC_CHK_DELAYFD2) == BST_CHECKED ? MSK_DELAY_FDSEEK : 0);

	config.wav_reverse = (IsDlgButtonChecked(hDlg, IDC_CHK_REVERSE) == BST_CHECKED);
	config.wav_half = (IsDlgButtonChecked(hDlg, IDC_CHK_HALFWAVE) == BST_CHECKED);
	config.wav_correct = !(IsDlgButtonChecked(hDlg, IDC_RADIO_NOCORR) == BST_CHECKED);
	config.wav_correct_type = (IsDlgButtonChecked(hDlg, IDC_RADIO_SINW) == BST_CHECKED ? 1 : 0);
	valuel = GetDlgItemInt(hDlg, IDC_TXT_CORRAMP0, &rc, false);
	if (rc == TRUE && 100 <= valuel && valuel <= 5000) {
		config.wav_correct_amp[0] = valuel;
	}
	valuel = GetDlgItemInt(hDlg, IDC_TXT_CORRAMP1, &rc, false);
	if (rc == TRUE && 100 <= valuel && valuel <= 5000) {
		config.wav_correct_amp[1] = valuel;
	}

	config.wav_sample_rate = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_SRATE, CB_GETCURSEL, 0, 0);
	config.wav_sample_bits = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_SBITS, CB_GETCURSEL, 0, 0);

	// I/O port address
	for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if ((1 << i) & IOPORT_MSK_ALL) {
			if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT1 + i) != BST_UNCHECKED)  {
				io_port |= (1 << i);
			} else {
				io_port &= ~(1 << i);
			}
		}
	}
	// crtc
#if defined(_MBS1)
	config.disptmg_skew = (int8_t)(SendDlgItemMessage(hDlg, IDC_COMBO_DISPTMG, CB_GETCURSEL, 0, 0) - 2);
	config.curdisp_skew = (int8_t)(SendDlgItemMessage(hDlg, IDC_COMBO_CURDISP, CB_GETCURSEL, 0, 0) - 2);
#else
	config.disptmg_skew = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_DISPTMG, CB_GETCURSEL, 0, 0);
	config.curdisp_skew = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_CURDISP, CB_GETCURSEL, 0, 0);
#endif

#ifdef USE_DIRECT3D
	// d3d use
	config.use_direct3d = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_D3D_USE, CB_GETCURSEL, 0, 0);

	// d3d filter type
	config.d3d_filter_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_D3D_FILTER, CB_GETCURSEL, 0, 0);
#endif
#ifdef USE_OPENGL
	// opengl use
	config.use_opengl = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_OPENGL_USE, CB_GETCURSEL, 0, 0);

	// opengl filter type
	config.gl_filter_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_OPENGL_FILTER, CB_GETCURSEL, 0, 0);
#endif

	// led show
	int led_show = (int)SendDlgItemMessage(hDlg, IDC_COMBO_LED_SHOW, CB_GETCURSEL, 0, 0);
	// led pos
	config.led_pos = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_LED_POS, CB_GETCURSEL, 0, 0);

	// capture type
	config.capture_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_CAPTURE_TYPE, CB_GETCURSEL, 0, 0);

	// snapshot path
	GetDlgItemText(hDlg, IDC_SNAP_PATH, buf, _MAX_PATH);
	config.snapshot_path.SetM(buf);
	// font file / path
	GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
	config.font_path.SetM(buf);
	// message font name
	GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf, _MAX_PATH);
	config.msgboard_msg_fontname.SetM(buf);
	GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf, _MAX_PATH);
	config.msgboard_info_fontname.SetM(buf);
	// message font size
	valuel = GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, &rc, TRUE);
	if (rc == TRUE && 1 <= valuel && valuel <= 60) {
		config.msgboard_info_fontsize = (uint8_t)valuel;
	}
	valuel = GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, &rc, TRUE);
	if (rc == TRUE && 1 <= valuel && valuel <= 60) {
		config.msgboard_msg_fontsize = (uint8_t)valuel;
	}

	// language
	valuel = (int)SendDlgItemMessage(hDlg, IDC_COMBO_LANGUAGE, CB_GETCURSEL, 0, 0);
	clocale->ChooseLocaleName(lang_list, valuel, config.language);

	// hostname, port
	for(int i=0; i<MAX_PRINTER; i++) {
		GetDlgItemText(hDlg, IDC_HOSTNAME_LPT0 + i, buf, _MAX_PATH);
		config.printer_server_host[i].Set(buf);
		valuel = GetDlgItemInt(hDlg, IDC_PORT_LPT0 + i, &rc, TRUE);
		if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
			config.printer_server_port[i] = valuel;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		GetDlgItemText(hDlg, IDC_HOSTNAME_COM0 + i, buf, _MAX_PATH);
		config.comm_server_host[i].Set(buf);
		valuel = GetDlgItemInt(hDlg, IDC_PORT_COM0 + i, &rc, TRUE);
		if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
			config.comm_server_port[i] = valuel;
		}
		valuel = (int)SendDlgItemMessage(hDlg, IDC_COMBO_COM0 + i, CB_GETCURSEL, 0, 0);
		if (valuel >= 0 && valuel <= 3) {
			config.comm_dipswitch[i] = valuel + 1;
		}
	}
#ifdef USE_DEBUGGER
	GetDlgItemText(hDlg, IDC_HOSTNAME_DBGR, buf, _MAX_PATH);
	config.debugger_server_host.Set(buf);
	valuel = GetDlgItemInt(hDlg, IDC_PORT_DBGR, &rc, TRUE);
	if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
		config.debugger_server_port = valuel;
	}
#endif

	// rom path
	GetDlgItemText(hDlg, IDC_ROM_PATH, buf, _MAX_PATH);
	config.rom_path.SetM(buf);
#if defined(_MBS1)
	// exram size
	exram_size_num = (int)SendDlgItemMessage(hDlg, IDC_COMBO_EXMEM, CB_GETCURSEL, 0, 0);
	// no wait
	config.mem_nowait = (IsDlgButtonChecked(hDlg, IDC_CHK_MEMNOWAIT) == BST_CHECKED);
#else
	// extended memory
	config.exram_size_num = IsDlgButtonChecked(hDlg, IDC_CHK_EXMEM) ? 1 : 0;
#endif

	// undef opcode
	config.show_dlg = (IsDlgButtonChecked(hDlg, IDC_CHK_UNDEFOP) == BST_CHECKED ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP);

#if defined(_MBS1) && defined(USE_Z80B_CARD)
	// z80bcard irq
	config.z80b_card_out_irq = (int)SendDlgItemMessage(hDlg, IDC_COMBO_Z80BCARD_IRQ, CB_GETCURSEL, 0, 0);
#endif

#if defined(_MBS1)
	// fmopn clock
	config.opn_clock = (int)SendDlgItemMessage(hDlg, IDC_COMBO_FMOPN, CB_GETCURSEL, 0, 0);
	// fmopn irq
	config.opn_irq = (int)SendDlgItemMessage(hDlg, IDC_COMBO_FMOPN_IRQ, CB_GETCURSEL, 0, 0);

	// use opn instead of psg
	config.use_opn_expsg = (IsDlgButtonChecked(hDlg, IDC_CHK_USE_OPN_EXPSG) == BST_CHECKED);

	emu->set_parami(VM::ParamSysMode, sys_mode);
	emu->set_parami(VM::ParamExMemNum, exram_size_num);
#endif

	emu->set_parami(VM::ParamFddType, fdd_type);
	emu->set_parami(VM::ParamIOPort, io_port);

	// message font
#ifdef USE_MESSAGE_BOARD
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
#endif

	gui->ChangeLedBox(led_show);
	gui->ChangeLedBoxPosition(config.led_pos);

	config.save();
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif
	emu->update_config();

	return (INT_PTR)TRUE;
}

#if 0
INT_PTR ConfigBox::onControlColorStatic(UINT message, WPARAM wParam, LPARAM lParam)
{
	HANDLE h = (HANDLE)GetStockObject(NULL_BRUSH);
	SetBkMode((HDC)wParam, TRANSPARENT);
	return (INT_PTR)h;
}
#endif

static int ctrl_ids0[] = {
	IDC_CHK_POWEROFF, IDC_STATIC_MODE_SW,
	ID_DIPSWITCH3, IDC_STATIC_1,
	IDC_STATIC_ALLOW, IDC_RADIO_NOFDD, IDC_RADIO_3FDD, IDC_RADIO_5FDD,
	IDC_STATIC_2,
#if defined(_MBS1)
	ID_DIPSWITCH1, ID_DIPSWITCH2,
	IDC_RADIO_5FDD1M,
#else
	IDC_RADIO_8FDD,
#endif
#ifdef USE_IOPORT_FDD
	IDC_STATIC_IOPORT1, IDC_STATIC_IOPORT2,
#endif
	IDC_STATIC_IOPORT3, IDC_STATIC_IOPORT4, IDC_STATIC_IOPORT5, IDC_STATIC_IOPORT6, IDC_STATIC_IOPORT7,
#if defined(_MBS1)
	IDC_STATIC_IOPORT8,
	IDC_STATIC_IOPORT9, IDC_STATIC_IOPORT10, IDC_STATIC_IOPORT11, IDC_STATIC_IOPORT12, IDC_STATIC_IOPORT13, IDC_STATIC_IOPORT14,
#if defined(USE_Z80B_CARD)
	IDC_STATIC_IOPORT17,
#endif
#endif
	IDC_STATIC_IOPORT15, IDC_STATIC_IOPORT16,
#ifdef USE_IOPORT_FDD
	IDC_CHK_IOPORT1, IDC_CHK_IOPORT2,
#endif
	IDC_CHK_IOPORT3, IDC_CHK_IOPORT4, IDC_CHK_IOPORT5, IDC_CHK_IOPORT6, IDC_CHK_IOPORT7,
#if defined(_MBS1)
	IDC_CHK_IOPORT8,
	IDC_CHK_IOPORT9, IDC_CHK_IOPORT10, IDC_CHK_IOPORT11, IDC_CHK_IOPORT12, IDC_CHK_IOPORT13, IDC_CHK_IOPORT14,
#if defined(USE_Z80B_CARD)
	IDC_CHK_IOPORT17,
#endif
#endif
	IDC_CHK_IOPORT15, IDC_CHK_IOPORT16,
	IDC_STATIC_3, IDC_STATIC_RESTART,
	-1
};
static int ctrl_ids1[] = {
#ifdef USE_DIRECT3D
	IDC_STATIC_D3D_USE, IDC_COMBO_D3D_USE, IDC_STATIC_D3D_FILTER, IDC_COMBO_D3D_FILTER, IDC_STATIC_7,
#endif
#ifdef USE_OPENGL
	IDC_STATIC_OPENGL_USE, IDC_COMBO_OPENGL_USE, IDC_STATIC_OPENGL_FILTER, IDC_COMBO_OPENGL_FILTER, IDC_STATIC_7,
#endif
	IDC_STATIC_LED_SHOW, IDC_COMBO_LED_SHOW, IDC_STATIC_LED_POS, IDC_COMBO_LED_POS,
	IDC_STATIC_CAPTURE_TYPE, IDC_COMBO_CAPTURE_TYPE,
	IDC_STATIC_DISPTMG, IDC_STATIC_CURDISP, IDC_COMBO_DISPTMG, IDC_COMBO_CURDISP, IDC_STATIC_6,
	IDC_STATIC_SNAP_PATH, IDC_SNAP_PATH, IDC_BTN_SNAP_PATH, IDC_STATIC_FONT_FILE, IDC_FONT_FILE, IDC_BTN_FONT_FILE,
	IDC_STATIC_MSG_FONT_NAME_L, IDC_MSG_FONT_NAME_L, IDC_STATIC_MSG_FONT_SIZE_L, IDC_MSG_FONT_SIZE_L, IDC_BTN_FONT_NAME_L,
	IDC_STATIC_MSG_FONT_NAME_S, IDC_MSG_FONT_NAME_S, IDC_STATIC_MSG_FONT_SIZE_S, IDC_MSG_FONT_SIZE_S, IDC_BTN_FONT_NAME_S,
	IDC_STATIC_LANGUAGE, IDC_COMBO_LANGUAGE, IDC_STATIC_RESTART_2,
	-1
};
static int ctrl_ids2[] = {
	IDC_CHK_REVERSE, IDC_CHK_HALFWAVE, IDC_STATIC_CORRECT, IDC_RADIO_NOCORR, IDC_RADIO_COSW, IDC_RADIO_SINW, IDC_STATIC_4,
	IDC_STATIC_CORRAMP0, IDC_TXT_CORRAMP0, IDC_STATIC_CORRAMP1, IDC_TXT_CORRAMP1,
	IDC_STATIC_SRATE, IDC_STATIC_SBITS, IDC_COMBO_SRATE, IDC_COMBO_SBITS, IDC_STATIC_5,
	IDC_STATIC_FDD_MOUNT, IDC_CHK_FDD_MOUNT0, IDC_CHK_FDD_MOUNT1, IDC_CHK_FDD_MOUNT2, IDC_CHK_FDD_MOUNT3,
	IDC_CHK_DELAYFD1, IDC_CHK_DELAYFD2, IDC_STATIC_8,
	-1
};
static int ctrl_ids3[] = {
	IDC_STATIC_NET0, IDC_STATIC_LPT0, IDC_STATIC_LPT1, IDC_STATIC_LPT2, IDC_STATIC_COM0, IDC_STATIC_COM1,
	IDC_HOSTNAME_NET0, IDC_HOSTNAME_LPT0, IDC_HOSTNAME_LPT1, IDC_HOSTNAME_LPT2, IDC_HOSTNAME_COM0, IDC_HOSTNAME_COM1,
	IDC_PORT_NET0, IDC_PORT_LPT0, IDC_PORT_LPT1, IDC_PORT_LPT2, IDC_PORT_COM0, IDC_PORT_COM1,
	IDC_COMBO_COM0, IDC_COMBO_COM1,
#ifdef USE_DEBUGGER
	IDC_STATIC_DBGR, IDC_HOSTNAME_DBGR, IDC_PORT_DBGR,
#endif
	-1
};
static int ctrl_ids4[] = {
	IDC_STATIC_ROM_PATH, IDC_ROM_PATH, IDC_BTN_ROM_PATH,
#if defined(_MBS1)
	IDC_COMBO_EXMEM, IDC_STATIC_EXMEM, IDC_STATIC_EXMEM_NOW,
	IDC_CHK_MEMNOWAIT,
#else
	IDC_CHK_EXMEM,
#endif
	IDC_CHK_UNDEFOP,
#if defined(_MBS1) && defined(USE_Z80B_CARD)
	IDC_STATIC_Z80BCARD_IRQ, IDC_COMBO_Z80BCARD_IRQ,
#endif
	IDC_STATIC_RESTART_3,
	-1
};
#if defined(_MBS1)
static int ctrl_ids5[] = {
	IDC_COMBO_FMOPN, IDC_STATIC_FMOPN, IDC_STATIC_FMOPN_2,
	IDC_COMBO_FMOPN_IRQ, IDC_STATIC_FMOPN_IRQ,
	IDC_CHK_USE_OPN_EXPSG,
	IDC_STATIC_RESTART_4,
	-1
};
#endif

void ConfigBox::select_tabctrl(int tab_num)
{
	HWND hCtrl;
	int cmdShow;

	selected_tabctrl = tab_num;
	for(int i=0; i<CONFIG_TABS; i++) {
		cmdShow = (selected_tabctrl == i ? SW_SHOW : SW_HIDE);
		switch(i) {
		case 0:
			// Mode
			for(int j=0; ctrl_ids0[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids0[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			if (selected_tabctrl == i) {
#if defined(_BML3MK5)
				// Mode switch
				hCtrl = GetDlgItem(hDlg, IDC_STATIC_MODE_SW);
				ShowWindow(hCtrl, (config.dipswitch & 4) != 0 ? SW_SHOWNORMAL : SW_HIDE);
#endif
				// I/O port address
				for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1 + i, (io_port & (1 << i)) != 0);
					hCtrl = GetDlgItem(hDlg, IDC_STATIC_IOPORT1 + i);
					ShowWindow(hCtrl, (config.io_port & (1 << i)) != 0 ? SW_SHOWNORMAL : SW_HIDE);
				}
#ifdef USE_IOPORT_FDD
				if (fdd_type == FDD_TYPE_3FDD) {
					// I/O port (3inch)
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
					EnableWindow(hCtrl, false);
					CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 1);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
					EnableWindow(hCtrl, false);
				}
				else if (fdd_type == FDD_TYPE_5FDD
#if defined(_MBS1)
					  || fdd_type == FDD_TYPE_5HFDD)
#else
					  || fdd_type == FDD_TYPE_8FDD)
#endif
				{
					// I/O port (5inch)
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
					EnableWindow(hCtrl, false);
					CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
					EnableWindow(hCtrl, false);
				}
#endif
			}
			break;
		case 1:
			// Screen
			for(int j=0; ctrl_ids1[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids1[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 2:
			// Tape
			for(int j=0; ctrl_ids2[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids2[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 3:
			// Network
			for(int j=0; ctrl_ids3[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids3[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 4:
			// Misc
			for(int j=0; ctrl_ids4[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids4[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
#if defined(_MBS1)
		case 5:
			// Sound
			for(int j=0; ctrl_ids5[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids5[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
#endif
		}
	}
}

}; /* namespace GUI_WIN */
