/** @file win_volumebox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2013.10.30 -

	@brief [ volume box ]
*/
#include "win_volumebox.h"
#include "../../emu.h"
#include "win_gui.h"

namespace GUI_WIN
{

VolumeBox::VolumeBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_VOLUME, new_font, new_emu, new_gui)
{
	int i = 0;
	memset(volumes, 0, sizeof(volumes));
	volumes[i++] = &config.volume;
	volumes[i++] = &config.beep_volume;
#if defined(_MBS1)
	volumes[i++] = &config.psg_volume;
	volumes[i++] = &config.psgexfm_volume;
	volumes[i++] = &config.psgexssg_volume;
	volumes[i++] = &config.opnfm_volume;
	volumes[i++] = &config.opnssg_volume;
#endif
	volumes[i++] = &config.psg6_volume;
	volumes[i++] = &config.psg9_volume;
	volumes[i++] = &config.relay_volume;
	volumes[i++] = &config.cmt_volume;
	volumes[i++] = &config.fdd_volume;

	i = 0;
	memset(mutes, 0, sizeof(mutes));
	mutes[i++] = &config.mute;
	mutes[i++] = &config.beep_mute;
#if defined(_MBS1)
	mutes[i++] = &config.psg_mute;
	mutes[i++] = &config.psgexfm_mute;
	mutes[i++] = &config.psgexssg_mute;
	mutes[i++] = &config.opnfm_mute;
	mutes[i++] = &config.opnssg_mute;
#endif
	mutes[i++] = &config.psg6_mute;
	mutes[i++] = &config.psg9_mute;
	mutes[i++] = &config.relay_mute;
	mutes[i++] = &config.cmt_mute;
	mutes[i++] = &config.fdd_mute;
}

VolumeBox::~VolumeBox()
{
}

INT_PTR VolumeBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	CDialogBox::onInitDialog(message, wParam, lParam);

	for(int i=0; i<VOLUMEBOX_NUMS; i++) {
		if (volumes[i]) {
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETRANGE, TRUE, MAKELPARAM(0, 100));
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETTICFREQ, 25, 0);
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETPOS  , TRUE, 100 - (*volumes[i]));
			SetVolumeText(i);
		}
		if (mutes[i]) {
			CheckDlgButton(hDlg, IDC_CHK_MUTE1+i, *mutes[i]);
		}
	}

	// layout
	SIZE siz;
	font->GetTextSize(hDlg, _T("0000000"), &siz);
	SIZE sli;
	GetControlSize(IDC_SLIDER_VOL1, &sli);

	CBox *box_all = new CBox(CBox::VerticalBox, 0, margin);
	CBox *box_hall;
	ShowDlgItem(IDC_STATIC_39, false);
	for(int i=0; i<VOLUMEBOX_NUMS; i++) {
		if (i == 7) {
			CBox *box_sp = new CBox(CBox::HorizontalBox, CBox::CenterPos);
			box_all->AddBox(box_sp);
			AdjustControl(box_sp, IDC_STATIC_39, 160, 1);
			ShowDlgItem(IDC_STATIC_39, true);
		}
		if (i == 0 || i == 7) {
			box_hall = new CBox(CBox::HorizontalBox, CBox::CenterPos);
			box_all->AddBox(box_hall);
		}
		CBox *box = new CBox(CBox::VerticalBox, 0, padding);
		box_hall->AddBox(box);
		AdjustControl(box, IDC_STATIC_1 + i, siz.cx, siz.cy * 2);
		AdjustControl(box, IDC_SLIDER_VOL1 + i, sli.cx, 100);
		AdjustControl(box, IDC_STATIC_21 + i, siz.cx, siz.cy);
		AdjustCheckBox(box, IDC_CHK_MUTE1 + i);

		if (i == 0) {
			CBox *box_sp = new CBox(CBox::HorizontalBox, CBox::MiddlePos, padding);
			box_hall->AddBox(box_sp);
			AdjustControl(box_sp, IDC_STATIC_0, 1, 160);
		}
	}

	// button
	CBox *box_btn = new CBox(CBox::HorizontalBox, CBox::RightPos, padding);
	box_all->AddBox(box_btn);
	AdjustButton(box_btn, IDOK, 8);

	box_all->Realize(*this);

	ResizeControl(IDC_STATIC_39, box_all->GetWidth() - 10, 1);

	delete box_all;

	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onHScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();
	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onVScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();
	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();

	return CDialogBox::onCommand(message, wParam, lParam);
}

void VolumeBox::SetVolume()
{
	for(int i=0; i<VOLUMEBOX_NUMS; i++) {
		if (volumes[i]) {
			*volumes[i] = (int)SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1 + i, TBM_GETPOS, 0, 0);
			*volumes[i] = 100 - (*volumes[i]);
			SetVolumeText(i);
		}
		if (mutes[i]) {
			*mutes[i] = (IsDlgButtonChecked(hDlg, IDC_CHK_MUTE1 + i) != 0);
		}
	}

	// callback
	if (emu) {
		emu->set_volume(0);
	}

	return;
}

void VolumeBox::SetVolumeText(int num)
{
	_TCHAR str[8];
	_stprintf(str, _T("%02d"), *volumes[num]);
	SetDlgItemText(hDlg, IDC_STATIC_21+num, str);
}

}; /* namespace GUI_WIN */
