/** @file windowmode.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2019.10.01

	@brief [ window mode ]
*/

#include "windowmode.h"
#include "../emu.h"
#include <stdlib.h>
//#include "../main.h"

static const VmSize window_proposal[] = {
	{640,400},{640,480},{800,600},{1024,768},{0,0}
};

WindowMode::WindowMode()
{
	window_mode_count = 0;
}

WindowMode::~WindowMode()
{
}

/// enumerate screen mode for window
/// calcurate magnify range
/// @param [in] max_width
/// @param [in] max_height
void WindowMode::Enum(int max_width, int max_height)
{
	// enumerate screen mode for window
	int w,h;
	bool minsize = false;
	bool maxsize = false;

	window_mode_count = 0;
	for (int i=0; window_proposal[i].w != 0 && window_mode_count < WINDOW_MODE_MAX; i++) {
		w = window_proposal[i].w;
		h = window_proposal[i].h;
		// minimum window size?
		if (w < MIN_WINDOW_WIDTH || h < MIN_WINDOW_HEIGHT) {
			continue;
		}
		if (!minsize) {
			w = MIN_WINDOW_WIDTH;
			h = MIN_WINDOW_HEIGHT;
			i--;
			minsize = true;
		}
		if (w > MAX_WINDOW_WIDTH || h > MAX_WINDOW_HEIGHT) {
			if (!maxsize) {
				w = MAX_WINDOW_WIDTH;
				h = MAX_WINDOW_HEIGHT;
				maxsize = true;
				i--;
			} else {
				break;
			}
		}
		for (int pow = 10; pow <= 30 && window_mode_count < WINDOW_MODE_MAX; pow+=5) {
			if ((w*pow/10) <= max_width && (h*pow/10) <= max_height) {
				bool found = false;
				for(int j = 0; j < window_mode_count; j++) {
					if(window_mode[j].power == pow && window_mode[j].width == (w*pow/10) && window_mode[j].height == (h*pow/10)) {
						found = true;
						break;
					}
				}
				if (!found) {
					window_mode[window_mode_count].power = pow;
					window_mode[window_mode_count].width = w * pow / 10;
					window_mode[window_mode_count].height = h * pow / 10;
					logging->out_debugf(_T("window_mode:%d %dx%d x%.1f"),window_mode_count, w * pow / 10, h * pow / 10, (double)pow / 10.0);
					window_mode_count++;
				}
			} else {
				break;
			}
		}
	}
	qsort(&window_mode, window_mode_count, sizeof(struct window_mode_st), qsort_by_w_h_n);
}

int WindowMode::qsort_by_w_h_n(const void *a, const void *b)
{
	struct window_mode_st *a1 = (struct window_mode_st *)a;
	struct window_mode_st *b1 = (struct window_mode_st *)b;

	int rc = 0;
	if (!rc) rc = (a1->width - b1->width);
	if (!rc) rc = (a1->height - b1->height);
	if (!rc) rc = (a1->power - b1->power);
	return rc;
}

const window_mode_t *WindowMode::Get(int num) const
{
	if (0 <= num && num < window_mode_count) {
		return &window_mode[num];
	}
	return NULL;
}

int WindowMode::Find(int width, int height) const
{
	int match = -1;
	for(int i = 0; i < window_mode_count; i++) {
		if (window_mode[i].width == width
		&&  window_mode[i].height == height) {
			match = i;
			break;
		}
	}
	return match;
}
